@@ -1,5 +1,5 @@
 #   Copyright (C) 2006-2007 Daniel Burrows
-#   Copyright (C) 2018 Scott Hansen
+#   Copyright (C) 2019 Scott Hansen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,7 +23,9 @@ import json
 import os
 from os.path import dirname, exists, expanduser
 import re
-from subprocess import Popen
+import shlex
+from subprocess import Popen, PIPE
+import sys
 from threading import Thread
 from time import sleep
 import webbrowser
@@ -90,7 +92,7 @@ def splittext(text, search, attr):
 class URLChooser:
 
     def __init__(self, extractedurls, compact=False, dedupe=False, shorten=True,
-                 run=""):
+                 run="", pipe=False):
         self.conf = expanduser("~/.config/urlscan/config.json")
         self.palettes = []
         try:
@@ -124,6 +126,7 @@ class URLChooser:
         self.shorten = shorten
         self.compact = compact
         self.run = run
+        self.pipe = pipe
         self.search = False
         self.search_string = ""
         self.no_matches = False
@@ -146,6 +149,7 @@ class URLChooser:
         header = (":: q - Quit :: "
                   "/ - search :: "
                   "c - context :: "
+                  "C - copy to clipboard :: "
                   "s - URL short :: "
                   "S - all URL short :: "
                   "g/G - top/bottom :: "
@@ -220,7 +224,7 @@ class URLChooser:
         return [i for i in keys if i != 'backspace']
 
     def unhandled(self, key):
-        """Add other keyboard actions (q, j, k, s, S, c, g, G) not handled by
+        """Add other keyboard actions (q, j, k, s, S, c, C, g, G) not handled by
         the ListBox widget.
 
         """
@@ -292,9 +296,9 @@ class URLChooser:
         elif key in ('S', 'u'):
             # Toggle all shortened or escaped URLs
             if key == 'S':
-                self.shorten = False if self.shorten is True else True
+                self.shorten = not self.shorten
             if key == 'u':
-                self.unesc = False if self.unesc is True else True
+                self.unesc = not self.unesc
                 self.urls, self.urls_unesc = self.urls_unesc, self.urls
             urls = iter(self.urls)
             for item in self.items:
@@ -315,7 +319,24 @@ class URLChooser:
             self.items, self.items_com = self.items_com, self.items
             self.top.body = urwid.ListBox(self.items)
             self.top.body.focus_position = self._cur_focus(fpo)
-            self.compact = False if self.compact is True else True
+            self.compact = not self.compact
+        elif key == 'C':
+            # Copy highlighted url to clipboard
+            fpo = self.top.body.focus_position
+            url_idx = len([i for i in self.items[:fpo + 1]
+                           if isinstance(i, urwid.Columns)]) - 1
+            if self.compact is False and fpo <= 1:
+                return
+            url = self.urls[url_idx]
+            cmds = ("xsel -i", "xclip -i")
+            for cmd in cmds:
+                try:
+                    proc = Popen(shlex.split(cmd), stdin=PIPE)
+                    proc.communicate(input=url.encode(sys.getdefaultencoding()))
+                    self._footer_start_thread("Copied url to primary selection", 5)
+                except OSError:
+                    continue
+                break
         elif key == 'p':
             # Loop through available palettes
             self.palette_idx += 1
@@ -462,8 +483,12 @@ class URLChooser:
                     self.enter = False
             elif not self.run:
                 webbrowser.open(url)
+            elif self.run and self.pipe:
+                process = Popen(shlex.split(self.run), stdout=PIPE, stdin=PIPE)
+                process.communicate(input=url.encode(sys.getdefaultencoding()))
             else:
                 Popen(self.run.format(url), shell=True).communicate()
+
             size = self.tui.get_cols_rows()
             self.draw_screen(size)
         return browse
